import React, { forwardRef } from 'react';
import { extend } from '@react-three/fiber';
import { Material } from 'three';

var keywords = {
  posiiton: "csm_Position",
  normal: "csm_Normal",
  pointSize: "csm_PointSize",
  diffuseColor: "csm_DiffuseColor"
};

const VERT = {
  "#include <defaultnormal_vertex>": `
    vec3 transformedNormal = ${keywords.normal};
    `,
  "#include <project_vertex>": `
    transformed = ${keywords.posiiton};
    #include <project_vertex>
  `,
  "gl_PointSize = size;": `gl_PointSize = ${keywords.pointSize};`
};
const FRAG = {
  "#include <color_fragment>": `
    #include <color_fragment>
    diffuseColor = ${keywords.diffuseColor};
`
};

class CustomShaderMaterial$1 extends Material {
  constructor(baseMaterial, fragmentShader, vertexShader, uniforms, opts) {
    // @ts-ignore
    const base = new baseMaterial(opts);
    super();

    for (const key in base) {
      // @ts-ignore
      if (this[key] === undefined) this[key] = 0; // @ts-ignore

      this[key] = base[key];
    }

    const parsedFragmentShdaer = this.parseShader(fragmentShader);
    const parsedVertexShdaer = this.parseShader(vertexShader);
    this.uniforms = uniforms || {};

    this.onBeforeCompile = shader => {
      if (parsedFragmentShdaer) {
        const patchedFragmentShdaer = this.patchShader(parsedFragmentShdaer, shader.fragmentShader, FRAG);
        shader.fragmentShader = patchedFragmentShdaer;
      }

      if (parsedVertexShdaer) {
        const patchedVertexShdaer = this.patchShader(parsedVertexShdaer, shader.vertexShader, VERT);
        shader.vertexShader = patchedVertexShdaer;
      }

      shader.uniforms = { ...shader.uniforms,
        ...this.uniforms
      };
      this.uniforms = shader.uniforms;
      this.needsUpdate = true;
    };

    this.needsUpdate = true;
  }

  patchShader(customShader, shader, patchMap) {
    let patchedShader = shader;
    Object.keys(patchMap).forEach(key => {
      const v = patchMap[key];
      patchedShader = replaceAll(patchedShader, key, v);
    });
    patchedShader = patchedShader.replace('void main() {', `
          ${customShader.header}
          void main() {
            vec3 csm_Position;
            vec3 csm_Normal;
            vec4 csm_DiffuseColor;
            float csm_PointSize;
            ${customShader.main}
          `);
    patchedShader = customShader.defines + patchedShader;
    return patchedShader;
  }

  parseShader(shader) {
    if (!shader) return;
    const parsedShader = {
      defines: '',
      header: '',
      main: ''
    };
    const main = shader.match(/void\s*main\s*\w*\s*\([\w\s,]*\)\s*{([\w\W]*?)}/g);

    if (main != null && main.length) {
      const mainBody = main[0].match(/\{((.|\n)*?)\}/g);

      if (mainBody != null && mainBody.length) {
        parsedShader.main = mainBody[0];
      }

      const rest = shader.replace(main[0], '');
      const defines = rest.match(/#(.*?;)/g) || [];
      const header = defines.reduce((prev, curr) => prev.replace(curr, ''), rest);
      parsedShader.header = header;
      parsedShader.defines = defines.join('\n');
    }

    return parsedShader;
  }

}

const replaceAll = (str, find, rep) => str.split(find).join(rep);

extend({
  CustomShaderMaterial_: CustomShaderMaterial$1
});
const CustomShaderMaterial = /*#__PURE__*/forwardRef(({
  baseMaterial,
  fragmentShader,
  vertexShader,
  uniforms,
  ...rest
}, ref) => {
  return /*#__PURE__*/React.createElement("customShaderMaterial_", {
    ref: ref,
    args: [baseMaterial, fragmentShader, vertexShader, uniforms, { ...rest
    }]
  });
});

export { CustomShaderMaterial as default };
